"""
Magics to have disqus conversation in the notebook.
"""

from metakernel import Magic
from IPython.display import HTML

class ConversationMagic(Magic):

    def cell_conversation(self, id):
        """
        %conversation ID - insert conversation by ID
        %%conversation ID - insert conversation by ID
        """
        html = """
<div id="disqus_thread"></div>
<script>
(function() { // DON'T EDIT BELOW THIS LINE
    var d = document, s = d.createElement('script');
    s.src = '//%s.disqus.com/embed.js';
    s.setAttribute('data-timestamp', +new Date());
    (d.head || d.body).appendChild(s);
})();
</script>
<noscript>Please enable JavaScript to view the <a href="https://disqus.com/?ref_noscript">comments powered by Disqus.</a></noscript>
""" % id
        self.kernel.Display(HTML(html))

    def line_conversation(self, id):
        """
        %conversation ID - insert conversation by ID
        %%conversation ID - insert conversation by ID
        """
        self.cell_conversation(id)
        self.evaluate = False

def register_magics(kernel):
    kernel.register_magics(ConversationMagic)

def register_ipython_magics():
    from metakernel import IPythonKernel
    from IPython.core.magic import register_line_magic, register_cell_magic
    kernel = IPythonKernel()
    magic = ConversationMagic(kernel)

    @register_line_magic
    def conversation(id):
        magic.line_conversation(id)

    @register_cell_magic
    def conversation(id, cell):
        magic.code = cell
        magic.cell_conversation(id)



# Copyright (c) Metakernel Development Team.
# Distributed under the terms of the Modified BSD License.

from metakernel import Magic
from IPython.display import HTML



class ProcessingMagic(Magic):
    canvas_id = 0

    def cell_processing(self, dummy=None):
        """
        %%processing - run the cell in the language Processing

        This cell magic will execute the contents of the cell as a
        Processing program. This uses the Java-based Processing
        language.
        
        Example:

            %%processing
            setup() {
            }
            draw() {
            }
        """
        self.canvas_id += 1
        """%%processing - run contents of cell as a Processing script"""

        repr_code = repr(self.code)
        if repr_code.startswith('u'):
            repr_code = repr_code[1:]

        env = {"code": repr_code,
               "id": self.canvas_id}
        code = """
<canvas id="canvas_%(id)s"></canvas>
<script>
require([window.location.protocol + "//calysto.github.io/javascripts/processing/processing.js"], function () {
    var processingCode = %(code)s;
    var cc = Processing.compile(processingCode);
    var processingInstance = new Processing("canvas_%(id)s", cc);
});
</script>
""" % env
        html = HTML(code)
        self.kernel.Display(html)
        self.evaluate = False

def register_magics(kernel):
    kernel.register_magics(ProcessingMagic)

def register_ipython_magics():
    from metakernel import IPythonKernel
    from IPython.core.magic import register_cell_magic
    kernel = IPythonKernel()
    magic = ProcessingMagic(kernel)

    @register_cell_magic
    def processing(line, cell):
        """
        """
        magic.code = cell
        magic.cell_processing()



# Copyright (c) Metakernel Development Team.
# Distributed under the terms of the Modified BSD License.

from metakernel import Magic


class MagicMagic(Magic):

    def line_magic(self, line):
        """
        %magic - show installed magics

        This line magic shows all of the install magics, either from
        the system magic folder, or your own private magic folder.

        """
        line_magics = []
        cell_magics = []

        for (name, magic) in self.kernel.line_magics.items():
            line_magics.append(magic.get_help('line', name).split("\n")[0])
        for (name, magic) in self.kernel.cell_magics.items():
            cell_magics.append(magic.get_help('cell', name).split("\n")[0])

        prefixes = self.kernel.magic_prefixes
        line_magics = ("\n    ".join(sorted(line_magics)))
        line_magics = line_magics.replace('%', prefixes['magic'])
        cell_magics = ("\n    ".join(sorted(cell_magics)))
        cell_magics = cell_magics.replace('%', prefixes['magic'])

        self.kernel.Print("Line magics:")
        self.kernel.Print("    " + line_magics)
        self.kernel.Print("")
        self.kernel.Print("Cell magics:")
        self.kernel.Print("    " + cell_magics)
        self.kernel.Print("")
        self.kernel.Print("Shell shortcut:")
        self.kernel.Print("    %s COMMAND ... - execute command in shell" % prefixes['shell'])
        self.kernel.Print("")
        self.kernel.Print(
            "Any cell magic can be made persistent for rest of session by using {0}{0}{0} prefix.".format(prefixes['magic']))
        self.kernel.Print("")
        self.kernel.Print("Help on items:")
        for string in self.kernel.line_magics['help'].help_strings():
            self.kernel.Print("    " + string)
        self.kernel.Print("")

    def get_magic(self, info, get_args=False):

        if not info['magic']:
            return None

        minfo = info['magic']
        name = minfo['name']
        if minfo['type'] == 'sticky':
            sname = (self.kernel.magic_prefixes['magic'] * 2) + name
            if sname in self.kernel.sticky_magics:
                del self.kernel.sticky_magics[sname]
                self.kernel.Print("%s removed from session magics.\n" % sname)
                # dummy magic to eat this line and continue:
                return Magic(self.kernel)
            else:
                self.kernel.sticky_magics[sname] = minfo['args']
                self.kernel.Print("%s added to session magics.\n" % name)

        cell_magics = self.kernel.cell_magics
        line_magics = self.kernel.line_magics
        if minfo['type'] in ['cell', 'sticky'] and name in cell_magics.keys():
            magic = cell_magics[name]
        elif minfo['type'] == 'line' and name in line_magics.keys():
            magic = line_magics[name]

        else:
            # FIXME: Raise an error
            return None
        if get_args:
            return magic.get_args(minfo['type'], minfo['name'],
                                  minfo['code'], minfo['args'])
        else:
            return magic.call_magic(minfo['type'], minfo['name'],
                                    minfo['code'], minfo['args'])


def register_magics(kernel):
    kernel.register_magics(MagicMagic)



import os
import re
import subprocess
from nose.tools import assert_raises

from metakernel import MetaKernel
from metakernel.tests.utils import (get_kernel, get_log_text, EvalKernel, 
                                    clear_log_text)

def test_magics():
    kernel = get_kernel()
    for magic in ['cd', 'connect_info', 'download', 'html', 'install_magic',
                  'javascript', 'latex', 'lsmagic', 'magic', 'plot',
                  'reload_magics', 'shell']:
        msg = "magic '%s' is not in line_magics" % magic
        assert magic in kernel.line_magics, msg

    for magic in ['file', 'html', 'javascript', 'latex', 'shell', 'time']:
        assert magic in kernel.cell_magics

    with open('TEST.txt', 'wb'):
        pass
    kernel.get_magic('%shell ls')
    log_text = get_log_text(kernel)
    assert 'TEST.txt' in log_text
    os.remove('TEST.txt')


def test_help():
    kernel = get_kernel()
    resp = kernel.get_help_on('%shell', 0)
    assert 'run the line as a shell command' in resp

    resp = kernel.do_execute('%cd?', False)
    assert 'change current directory of session' in resp[
        'payload'][0]['data']['text/plain']

    resp = kernel.get_help_on('what', 0)
    assert resp == "Sorry, no help is available on 'what'.", ("response was actually %s" % resp)


def test_complete():
    kernel = get_kernel()
    comp = kernel.do_complete('%connect_', len('%connect_'))
    assert comp['matches'] == ['%connect_info'], str(comp['matches'])

    comp = kernel.do_complete('%%fil', len('%%fil'))
    assert comp['matches'] == ['%%file'], str(comp['matches'])

    comp = kernel.do_complete('%%', len('%%'))
    assert '%%file' in comp['matches']
    assert '%%html' in comp['matches']


def test_inspect():
    kernel = get_kernel()
    kernel.do_inspect('%lsmagic', len('%lsmagic'))
    log_text = get_log_text(kernel)
    assert "list the current line and cell magics" in log_text

    kernel.do_inspect('%lsmagic ', len('%lsmagic') + 1)


def test_path_complete():
    kernel = get_kernel()
    comp = kernel.do_complete('~/.ipytho', len('~/.ipytho'))
    assert comp['matches'] == ['ipython/']

    paths = [p for p in os.listdir(os.getcwd())
             if not p.startswith('.') and not '-' in p]

    for path in paths:
        comp = kernel.do_complete(path, len(path) - 1)

        if os.path.isdir(path):
            path = path.split()[-1]
            assert path + os.sep in comp['matches'], ("'%s' not in '%s'" % (path + os.sep, comp['matches']))
        else:
            path = path.split()[-1]
            assert path in comp['matches'], (comp['matches'], path)


def test_ls_path_complete():
    kernel = get_kernel()
    comp = kernel.do_complete('! ls ~/.ipytho', len('! ls ~/.ipytho'))
    assert comp['matches'] == ['ipython/'], comp


def test_history():
    kernel = get_kernel()
    kernel.do_execute('!ls', False)
    kernel.do_execute('%cd ~', False)
    kernel.do_shutdown(False)

    with open(kernel.hist_file, 'rb') as fid:
        text = fid.read().decode('utf-8', 'replace')

    assert '!ls' in text
    assert '%cd' in text

    kernel = get_kernel()
    kernel.do_history(None, None, None)
    assert '!ls' in ''.join(kernel.hist_cache)
    assert '%cd ~'


def test_sticky_magics():
    kernel = get_kernel()
    kernel.do_execute('%%%html\nhello', None)
    text = get_log_text(kernel)

    assert 'html added to session magics' in text
    kernel.do_execute('<b>hello</b>', None)
    kernel.do_execute('%%%html', None)
    text = get_log_text(kernel)
    assert text.count('Display Data') == 2
    assert 'html removed from session magics' in text


def test_shell_partial_quote():
    kernel = get_kernel()
    kernel.do_execute('%cd "/home/', False)
    text = get_log_text(kernel)
    assert """No such file or directory: '"/home/'""" in text, text

def test_other_kernels():
    class SchemeKernel(MetaKernel):
        help_suffix = {}
        def do_execute_direct(self, code):
            return "OK"

    kernel = get_kernel(SchemeKernel)
    resp = kernel.do_execute('dir?', None)
    assert len(resp['payload']) == 0, "should handle this, rather than using help"
    resp = kernel.do_execute('?dir?', None)
    assert len(resp['payload']) == 1, "should use help"
    message = resp['payload'][0]['data']['text/plain']
    assert "Sorry, no help is available on 'dir?'." == message, message

    content = kernel.do_inspect('dir', len('dir'))
    assert (content['status'] == 'aborted' and 
            content['found'] == False), "do_inspect should abort, and be not found"

    content = kernel.do_inspect('len(dir', len('len(dir'))
    assert (content['status'] == 'aborted' and 
            content['found'] == False), "do_inspect should abort, and be not found"

    content = kernel.do_inspect('(dir', len('(dir'))
    assert (content['status'] == 'aborted' and 
            content['found'] == False), "do_inspect should abort, and be not found"

    # Now change it so that there is help available:
    kernel.get_kernel_help_on = lambda info, level=0, none_on_fail=False: \
                                   "Help is available on '%s'." % info['obj']
    content = kernel.do_inspect('dir', len('dir'))
    message = content["data"]["text/plain"]
    match = re.match("Help is available on '(.*)'", message)
    assert match.groups()[0] == "dir", message + " for 'dir'"

    content = kernel.do_inspect('len(dir', len('len(dir'))
    message = content["data"]["text/plain"]
    match = re.match("Help is available on '(.*)'", message)
    assert match.groups()[0] == "dir", message + " for 'dir'"

    content = kernel.do_inspect('(dir', len('(dir'))
    message = content["data"]["text/plain"]
    match = re.match("Help is available on '(.*)'", message)
    assert match.groups()[0] == "dir", message + " for 'dir'"


def test_do_execute_meta():
    kernel = get_kernel(EvalKernel)
    kernel.do_execute("~~META~~: reset")
    text = get_log_text(kernel)
    assert "RESET" in text, text
    clear_log_text(kernel)

    kernel.do_execute("~~META~~: stop")
    text = get_log_text(kernel)
    assert "STOP" in text, text
    clear_log_text(kernel)

    kernel.do_execute("~~META~~: step")
    text = get_log_text(kernel)
    assert "STEP" in text, text
    clear_log_text(kernel)

    kernel.do_execute("~~META~~: inspect something")
    text = get_log_text(kernel)
    assert "INSPECT" in text, text
    clear_log_text(kernel)


def test_do_execute_meta2():
    kernel = get_kernel()
    for code in ['reset, stop', 'step', 'inspect ', 'garbage']:
        assert_raises(Exception, kernel.do_execute, "~~META~~: %s" % code)


def test_misc():
    class TestKernel(MetaKernel):
        def do_execute_file(self, filename):
            self.Print("This language does not support running files")

        def do_function_direct(self, f, arg):
            self.Print("%s(%s)" % (f, self.repr(arg)))

        def repr(self, arg):
            return "XXX"

    kernel = get_kernel(TestKernel)
    assert kernel.do_execute_direct('garbage') is None
    kernel.do_execute_file('hello.txt')
    assert "This language does not support" in get_log_text(kernel)

    clear_log_text(kernel)

    kernel.do_function_direct('hello', 'world')
    text = get_log_text(kernel)
    assert "hello(XXX)" in text, text
    kernel.restart_kernel()

    ret = kernel.do_is_complete('hello\n')
    assert ret == {'status': 'complete'}

    assert kernel.do_inspect('hello', 10) is None

def teardown():
    if os.path.exists("TEST.txt"):
        os.remove("TEST.txt")




from metakernel.tests.utils import (get_kernel, EvalKernel, 
                                    get_log_text, clear_log_text)

def test_run_magic():
    kernel = get_kernel(EvalKernel)
    kernel.do_execute("%%run %s" % __file__.replace(".pyc", ".py"))
    kernel.do_execute("TEST")
    text = get_log_text(kernel)
    assert '42' in text, "Didn't run this file"

    clear_log_text(kernel)
    kernel.do_execute("%%run --language python %s" % __file__.replace(".pyc", ".py"))
    kernel.do_execute("TEST")
    text = get_log_text(kernel)
    assert '42' in text, "Didn't run this file"

TEST = 42




from metakernel.tests.utils import get_kernel, get_log_text

def test_reload_magics_magic():
    kernel = get_kernel()
    kernel.do_execute("%reload_magics")
    text = get_log_text(kernel)

    for item in "%cd %connect_info %download %edit %help %html %install_magic %javascript %kernel %kx %latex %load %lsmagic %magic %parallel %plot %pmap %px %python %reload_magics %restart %run %shell %macro %%debug %%file %%help %%html %%javascript %%kx %%latex %%processing %%px %%python %%shell %%show %%macro %%time".split():
        assert item in text, ("load_magic didn't list '%s'" % item)




# Copyright (c) Metakernel Development Team.
# Distributed under the terms of the Modified BSD License.

from metakernel import Magic, option
import inspect
import ast
import os

class MacroMagic(Magic):
    macros = {"renumber-cells": """%%javascript

        var cells = IPython.notebook.get_cells();
        // We only keep the code cells.
        cells = cells.filter(function(c)
            {
                return c instanceof IPython.CodeCell; 
            })
        // We set the input prompt of all code cells.
        for (var i = 0; i < cells.length; i++) {
            cells[i].set_input_prompt(i + 1);
        }
"""}

    def __init__(self, *args, **kwargs):
        super(MacroMagic, self).__init__(*args, **kwargs)
        self._load_macros()

    @option(
        '-d', '--delete', action='store_true', default=False,
        help='delete a named macro'
    )
    @option(
        '-l', '--list', action='store_true', default=False,
        help='list macros'
    )
    @option(
        '-s', '--show', action='store_true', default=False,
        help='show macro'
    )
    def line_macro(self, name, delete=False, list=False, show=False):
        """
        %macro NAME - execute a macro
        %macro -l [all|learned|system] - list macros
        %macro [-s] [-d] NAME - show or delete a macro

        This line macro will execute, show, list, or delete the 
        named macro.

        Examples:
            %macro renumber-cells

            %%macro test
            print "Ok!"

            %macro -l all

            %macro -d test
        """
        if list:
            self._list_macros(name)
        elif show:
            retval = "%%%%macro %s\n" % name
            if name in self.macros:
                retval += self.macros[name]
            elif name in self.learned:
                retval += self.learned[name]
            self.kernel.Print(retval)
        elif name in self.learned:
            if delete:
                del self.learned[name]
                self._save_macros()
            else:
                self.code = self.code.strip()
                if self.code: 
                    self.code += "\n"
                self.code += self.learned[name]
        elif name in self.macros:
            if delete:
                raise Exception("Can't delete system macro")
            else:
                self.code = self.code.strip()
                if self.code: 
                    self.code += "\n"
                self.code += self.macros[name]
        elif name == "":
            self._list_macros()
        else:
            self._list_macros(retval="No such macro: '%s'\n\n" % name,
                              error=True)
        self.evaluate = True

    def _list_macros(self, name="all", retval="", error=False):
        retval += "Available macros:\n"
        if name in ["all", "system"]:
            retval += "    System:\n"
            for macro in sorted(self.macros.keys()):
                retval += "        %s\n" % macro
        if name in ["all", "learned"]:
            retval += "    Learned:\n"
            for macro in sorted(self.learned.keys()):
                retval += "        %s\n" % macro
        if error:
            self.kernel.Error(retval)
        else:
            self.kernel.Print(retval)

    def _load_macros(self):
        self.learned = {}
        local_macros_dir = self.kernel.get_local_magics_dir()
        # Search all of the places there could be macros:
        files = [os.path.join(os.path.dirname(os.path.abspath(__file__)), "macros.json"),
                 os.path.join(os.path.dirname(os.path.abspath(inspect.getfile(self.__class__))), "macros.json"),
                 os.path.join(local_macros_dir, "macros.json")]
        for macro_file in files:
            try:
                data = ast.literal_eval(open(macro_file).read()) 
            except:
                continue
            self.learned.update(data)

    def _save_macros(self):
        local_macros_dir = self.kernel.get_local_magics_dir()
        filename = os.path.join(local_macros_dir, "macros.json")
        open(filename, "w").write(str(self.learned))

    def cell_macro(self, name):
        """
        %%macro NAME - learn a new macro

        This cell macro will learn the macro in the
        cell. The cell contents are just commands (macros
        or code in the kernel language).

        Example:
            %%macro test
            print "Ok!"

            %macro test
            Ok!
        """
        self.learned[name] = self.code
        self._save_macros()
        self.evaluate = False

def register_magics(kernel):
    kernel.register_magics(MacroMagic)





import os
from metakernel.tests.utils import (get_kernel, get_log_text,
                                    clear_log_text)


def test_ls_magic():
    kernel = get_kernel()
    kernel.do_execute("%ls /tmp")
    text = get_log_text(kernel)
    ## FIXME: failing on Travis
    #assert '/tmp/' in text, text[:100]
    clear_log_text(kernel)

    kernel.do_execute("%ls /tmp --recursive")
    text = get_log_text(kernel)
    ## FIXME: failing on Travis
    #assert '/tmp' in text, text[:100]
    clear_log_text(kernel)



# Copyright (c) Metakernel Development Team.
# Distributed under the terms of the Modified BSD License.

from metakernel import Magic
import urllib
try:
    import urlparse
except ImportError:
    from urllib import parse as urlparse
import os

try:
    urllib.URLopener
    def download(url, filename):
        opener = urllib.URLopener()
        opener.retrieve(url, filename)
except: # python3
    def download(url, filename):
        g = urllib.request.urlopen(url)
        with open(filename, 'wb') as f:
            f.write(g.read())

class InstallMagicMagic(Magic):

    def line_install_magic(self, url):
        """
        %install_magic URL - download and install magic from URL

        This line magic will copy the file at the URL into your 
        personal magic folder.

        Example:
            %install_magic http://path/to/some/magic.py

        """
        parts = urlparse.urlsplit(url)
        #('http', 'example.com', '/somefile.zip', '', '')
        path = parts[2]
        filename = os.path.basename(path)
        magic_filename = os.path.join(self.kernel.get_local_magics_dir(), filename)
        try:
            download(url, magic_filename)
            self.kernel.Print("Downloaded '%s'." % magic_filename)
            self.code = "%reload_magics\n" + self.code
        except Exception as e:
            self.kernel.Error(str(e))

def register_magics(kernel):
    kernel.register_magics(InstallMagicMagic)





from metakernel.tests.utils import (get_kernel, get_log_text, 
                                    clear_log_text, EvalKernel)

def test_show_magic():
    kernel = get_kernel(EvalKernel)
    results = kernel.do_execute("""%%show
Welcome to the big show!
""")
    text = results["payload"][0]["data"]["text/plain"]
    assert "Welcome to the big show!" in text, text

    results = kernel.do_execute("""%%show --output
# Welcome to the big show!
retval = "This is a test"
""")
    text = results["payload"][0]["data"]["text/plain"]
    assert "Welcome to the big show!" not in text, text
    assert "This is a test" in text, text



# Copyright (c) Metakernel Development Team.
# Distributed under the terms of the Modified BSD License.

try:
    from ipywidgets import widgets
except ImportError:
    widgets = None
from metakernel import Magic, option
import os
import getpass
import datetime

def touch(fname, times=None):
    with open(fname, 'a'):
        os.utime(fname, times)

class Activity(object):
    def __init__(self):
        self.questions = []
        self.filename = None
        self.results_filename = None
        self.instructors = []
        self.show_initial = True
        self.last_id = None

    def load(self, filename):
        if filename.startswith("~"):
            filename = os.path.expanduser(filename)
        filename = os.path.abspath(filename)
        self.filename = filename
        with open(self.filename) as fp:
            json_text = "".join(fp.readlines())
        self.load_json(json_text)
        if self.results_filename is None:
            self.results_filename = filename + ".results"
        touch(self.results_filename)

    def load_json(self, json_text):
        # Allow use of widgets:
        if widgets is None:
            return
        json = eval(json_text.strip(), {key: getattr(widgets, key) for key in dir(widgets)})
        if json.get("results_filename", None):
            self.results_filename = json["results_filename"]
        if json.get("instructors", []):
            for instructor in json["instructors"]:
                self.instructors.append(instructor)
        if json["activity"] == "poll":
            self.index = 0
            for item in json["items"]:
                if item["type"] == "multiple choice":
                    # FIXME: allow widgets; need to rewrite create/show:
                    question = item["question"]
                    #if isinstance(question, str):
                    #    question = widgets.HTML(question)
                    options = item["options"]
                    #for pos in range(len(options)):
                    #    option = options[pos]
                    #    if isinstance(option, str):
                    #        options[pos] = widgets.HTML(option)
                    q = Question(item["id"], question, options)
                    self.questions.append(q)
                else:
                    raise Exception("not a valid question 'type': use ['multiple choice']")
            self.create_widget()
            self.use_question(self.index)
        else:
            raise Exception("not a valid 'activity': use ['poll']")

    def use_question(self, index):
        self.set_question(self.questions[index].question)
        self.set_id(self.questions[index].id)
        self.results_html.layout.visibility = "hidden"
        self.results_button.layout.visibility = "visible" if (getpass.getuser() in self.instructors) else "hidden"
        self.prev_button.disabled = index == 0
        self.next_button.disabled = index == len(self.questions) - 1
        for i in range(5):
            self.choice_row_list[i].layout.visibility = "hidden"
            self.buttons[i].layout.visibility = "hidden"
        for i in range(len(self.questions[index].options)):
            self.choice_widgets[i].value = self.questions[index].options[i]
            self.choice_row_list[i].layout.visibility = "visible"
            self.buttons[i].layout.visibility = "visible"

    def create_widget(self):
        self.id_widget = widgets.HTML("")
        self.question_widget = widgets.HTML("")
        self.choice_widgets = []
        self.choice_row_list = []
        for count in range(1, 5 + 1):
            self.choice_widgets.append(widgets.HTML(""))
            self.choice_row_list.append(widgets.HBox([widgets.HTML("<b>%s</b>)&nbsp;&nbsp;" % count),
                                                      self.choice_widgets[-1]]))
        self.buttons = []
        for i in range(1, 5 + 1):
            button = widgets.Button(description = str(i))
            button.on_click(self.handle_submit)
            button.layout.margin = "20px"
            self.buttons.append(button)
        self.respond_row_widgets = widgets.HBox([widgets.HTML("""<br/><br clear="all"/><b>Respond</b>: """)] + self.buttons)
        self.next_button = widgets.Button(description="Next")
        self.next_button.on_click(self.handle_next)
        self.results_button = widgets.Button(description="Results")
        self.results_button.on_click(self.handle_results)
        self.prev_button = widgets.Button(description="Previous")
        self.prev_button.on_click(self.handle_prev)
        self.results_html = widgets.HTML("")
        self.top_margin = widgets.HTML("")
        #self.top_margin.layout.height = "100px"
        right_stack = widgets.VBox([self.top_margin, self.results_html])
        self.stack = widgets.VBox([self.id_widget, self.question_widget] + self.choice_row_list +
                                  [self.respond_row_widgets,
                                   widgets.HBox([self.prev_button, self.results_button, self.next_button])])
        self.output = widgets.Output()
        self.top_level = widgets.VBox([widgets.HBox([self.stack, right_stack]),
                                       self.output])

    def set_question(self, question):
        self.question_widget.value = "<h1>%s</h1>" % question

    def set_id(self, id):
        self.id_widget.value = "<p><b>Question ID</b>: %s</p>" % id
        self.id = id

    def handle_results(self, sender):
        # write out when we show the Results:
        self.handle_submit(sender)
        if self.last_id == self.questions[self.index].id:
            self.show_initial = not self.show_initial
        else:
            self.show_initial = True
            self.last_id = self.questions[self.index].id
        data = {}
        with open(self.results_filename) as fp:
            line = fp.readline()
            while line:
                if "::" in line:
                    id, user, time, choice = line.split("::")
                    if self.questions[self.index].id == id:
                        if choice.strip() != "Results":
                            if self.show_initial:
                                if user.strip() not in data:
                                    data[user.strip()] = choice.strip()
                            else: # shows last
                                data[user.strip()] = choice.strip()
                line = fp.readline()
        choices = {str(i): 0 for i in range(1, len(self.questions[self.index].options) + 1)}
        for datum in data.values():
            if datum not in choices:
                choices[datum] = 0
            choices[datum] += 1
        barvalues = [int(value) for key,value in sorted(choices.items())]
        self.stack.layout.width = "55%"
        try:
            from calysto.graphics import BarChart
            barchart = BarChart(size=(300, 400), data=barvalues, labels=sorted(choices.keys()))
            self.results_html.value = str(barchart)
            self.results_html.layout.visibility = "visible"
        except:
            with self.output:
                print(sorted(choices.keys()))
                print(barvalues)

    def handle_submit(self, sender):
        import fcntl
        with open(self.results_filename, "a+") as g:
            fcntl.flock(g, fcntl.LOCK_EX)
            g.write("%s::%s::%s::%s\n" % (self.id, getpass.getuser(), datetime.datetime.today(), sender.description))
            fcntl.flock(g, fcntl.LOCK_UN)
        self.output.clear_output()
        with self.output:
            print("Received: " + sender.description)

    def handle_next(self, sender):
        if self.index < len(self.questions) - 1:
            self.index += 1
            self.use_question(self.index)
            self.output.clear_output()

    def handle_prev(self, sender):
        if self.index > 0:
            self.index -= 1
            self.use_question(self.index)
            self.output.clear_output()

    def render(self):
        from metakernel.display import display
        display(self.top_level)

class Question(object):
    def __init__(self, id, question, options):
        self.id = id
        self.question = question
        self.options = options

class ActivityMagic(Magic):

    def line_activity(self, filename, mode=None):
        """
        %activity FILENAME - run a widget-based activity
          (poll, classroom response, clicker-like activity)

        This magic will load the JSON in the filename.

        Examples:
            %activity /home/teacher/activity1
            %activity /home/teacher/activity1 new
            %activity /home/teacher/activity1 edit
        """
        from IPython import get_ipython
        if mode == "new":
            text = '''
{"activity": "poll",
 "instructors": ["YOUR ID HERE"],
 "items": [
      {"id": "1",
       "question":  """When it comes to learning, metacognition (e.g., thinking about thinking) can be just as important as intelligence.""",
       "type": "multiple choice",
       "options": ["True", "False"]
      },
      {"id": "2",
       "question":  """What is the best way to learn from some text?""",
       "type": "multiple choice",
       "options": ["Read and reread the text.",
                   "Explain key ideas of the text to yourself while reading.",
                   "Underline key concepts.",
                   "Use a highlighter"]
      },
      {"id": "3",
       "question":  """Intelligence is fixed at birth.""",
       "type": "multiple choice",
       "options": ["True", "False"]
      },
      {"id": "4",
       "question":  """You have a test coming up. What's the best way to review the material?""",
       "type": "multiple choice",
       "options": ["Circle key points in the textbook.",
                   "Review relevant points of the lecture in audio format.",
                   "Take an informal quiz based on the material."]
      },
      {"id": "5",
       "question":  """To which of the following should you not tailor your learning?""",
       "type": "multiple choice",
       "options": ["Learning styles (visual, audio, etc.)", "Previous knowledge", "Interests", "Ability"]
      },
      {"id": "6",
       "question":  """Learning should be spaced out over time.""",
       "type": "multiple choice",
       "options": ["True", "False"]
      },
      {"id": "7",
       "question":  """Right-brained people learn differently from left-brained people.""",
       "type": "multiple choice",
       "options": ["True", "False"]
      }
   ]
}'''
            get_ipython().set_next_input(("%%%%activity %s\n\n" % filename) + text)
            return
        elif mode == "edit":
            text = "".join(open(filename, "r").readlines())
            get_ipython().set_next_input(("%%%%activity %s\n\n" % filename) + text)
        else:
            activity = Activity()
            activity.load(filename)
            activity.render()

    def cell_activity(self, filename):
        """
        %%activity FILENAME - make an activity from
          a JSON structure

        This magic will construct a Python file from the cell's
        content, a JSON structure.

        Example:
            %%activity /home/teacher/activity1
            {"activity": "poll",
             "instructors": ["teacher01"],
             "results_file": "/home/teacher/activity1.results",
             "items": [{"id": "...",
                        "type": "multiple choice",
                        "question": "...",
                        "options": ["...", ...]
                       }, ...]
            }

        In this example, users will load
        /home/teacher/activity1
        """
        if filename.startswith("~"):
            filename = os.path.expanduser(filename)
        filename = os.path.abspath(filename)
        with open(filename, "w") as fp:
            fp.write(self.code)
        activity = Activity()
        activity.load(filename)
        # Make sure results file is writable:
        os.chmod(activity.results_filename, 0o777)
        # Ok, let's test it (MetaKernel):
        self.line_activity(filename)
        self.evaluate = False

def register_magics(kernel):
    kernel.register_magics(ActivityMagic)

def register_ipython_magics():
    from metakernel import IPythonKernel
    from metakernel.utils import add_docs
    from IPython.core.magic import register_line_magic, register_cell_magic
    kernel = IPythonKernel()
    magic = ActivityMagic(kernel)
    # Make magics callable:
    kernel.line_magics["activity"] = magic
    kernel.cell_magics["activity"] = magic

    @register_line_magic
    @add_docs(magic.line_activity.__doc__)
    def activity(line):
        kernel.call_magic("%activity " + line)

    @register_cell_magic
    @add_docs(magic.cell_activity.__doc__)
    def activity(line, cell):
        magic.code = cell
        magic.cell_activity(line)



# Copyright (c) Metakernel Development Team.
# Distributed under the terms of the Modified BSD License.

from IPython.core.magic import magic_escapes
from metakernel import Magic
import os

class LSMagicMagic(Magic):

    def line_lsmagic(self):
        """
        %lsmagic - list the current line and cell magics
        
        This line magic will list all of the available cell and line
        magics installed in the system and in your personal magic
        folder.

        Example:
            %lsmagic
        """
        mesc = magic_escapes['line']
        cesc = magic_escapes['cell']

        line_magics = self.kernel.line_magics.keys()
        cell_magics = self.kernel.cell_magics.keys()

        mp = self.kernel.magic_prefixes['magic']
        out = [
            'Available line magics:',
            '  '.join(sorted([(mp + lm) for lm in line_magics])),
            '',
            'Available cell magics:',
            '  '.join(sorted([(mp + mp + cm) for cm in cell_magics])),
        ]
        self.kernel.Print('\n'.join(out))

def register_magics(kernel):
    kernel.register_magics(LSMagicMagic)



# Copyright (c) Metakernel Development Team.
# Distributed under the terms of the Modified BSD License.

from metakernel import Magic

class ReloadMagicsMagic(Magic):

    def line_reload_magics(self):
        """
        %reload_magics - reload the magics from the installed files

        Example:
            %reload_magics

        This line magic will reload the magics installed in the
        system, and in your private magic folder.

        You only need to do this if you edit a magic file. It runs
        automatically if you install a new magic.
        """
        self.kernel.reload_magics()
        self.code = "%lsmagic\n" + self.code

def register_magics(kernel):
    kernel.register_magics(ReloadMagicsMagic)




from metakernel.tests.utils import (get_kernel, get_log_text, 
                                    clear_log_text, EvalKernel)
import os
from nose.plugins.attrib import attr


@attr('network')
def test_jigsaw_magic():
    kernel = get_kernel(EvalKernel)
    kernel.do_execute("%jigsaw Processing --workspace workspace1")
    text = get_log_text(kernel)
    assert os.path.isfile("workspace1.html"), "File does not exist: workspace1.html"

def teardown():
    for fname in ['workspace1.html']:
        try:
            os.remove(fname)
        except:
            pass



from __future__ import absolute_import
from . import MetaKernel
from pexpect import EOF
from .replwrap import REPLWrapper, bash
from subprocess import check_output
import re

__version__ = '0.0'

version_pat = re.compile(r'version (\d+(\.\d+)+)')


class TextOutput(object):
    """Wrapper for text output whose repr is the text itself.

    This avoids `repr(output)` adding quotation marks around already-rendered
    text.
    """

    def __init__(self, output):
        self.output = output

    def __repr__(self):
        return self.output


class ProcessMetaKernel(MetaKernel):
    implementation = 'process_kernel'
    implementation_version = __version__
    language = 'process'
    language_info = {
        # 'mimetype': 'text/x-python',
        # 'language': 'python',
        # ------ If different from 'language':
        # 'codemirror_mode': 'language',
        # 'pygments_lexer': 'language',
        # 'file_extension': 'py',
    }

    @property
    def language_version(self):
        m = version_pat.search(self.banner)
        return m.group(1)

    _banner = "Process"

    @property
    def banner(self):
        return self._banner

    def __init__(self, *args, **kwargs):
        MetaKernel.__init__(self, *args, **kwargs)
        self.wrapper = None
        self.wrapper = self.makeWrapper()

    def do_execute_direct(self, code, silent=False):
        """Execute the code in the subprocess.
        """
        self.payload = []
        wrapper = self.wrapper
        child = wrapper.child

        if not code.strip():
            self.kernel_resp = {
                'status': 'ok',
                'execution_count': self.execution_count,
                'payload': [],
                'user_expressions': {},
            }
            return

        interrupted = False
        output = ''
        stream_handler = self.Print if not silent else None
        try:
            output = wrapper.run_command(code.rstrip(), timeout=None,
                                         stream_handler=stream_handler,
                                         stdin_handler=self.raw_input)
        except KeyboardInterrupt as e:
            interrupted = True
            output = wrapper.interrupt()
        except EOF:
            self.Print(child.before)
            self.do_shutdown(True)
            return

        if interrupted:
            self.kernel_resp = {
                'status': 'abort',
                'execution_count': self.execution_count,
            }

        exitcode, trace = self.check_exitcode()

        if exitcode:
            self.kernel_resp = {
                'status': 'error',
                'execution_count': self.execution_count,
                'ename': '', 'evalue': str(exitcode),
                'traceback': trace,
            }
        else:
            self.kernel_resp = {
                'status': 'ok',
                'execution_count': self.execution_count,
                'payload': [],
                'user_expressions': {},
            }

        if output:
            if stream_handler:
                stream_handler(output)
            else:
                return TextOutput(output)

    def check_exitcode(self):
        """
        Return (1, ["trace"]) if error.
        """
        return (0, None)

    def makeWrapper(self):
        """
        In this method the REPLWrapper is created and returned.
        REPLWrapper takes the name of the executable, and arguments
        describing the executable prompt:

        return REPLWrapper('bash', orig_prompt, prompt_change,
                           prompt_cmd=prompt_cmd,
                           extra_init_cmd=extra_init_cmd)

        The parameters are:

        :param orig_prompt: What the original prompt is (or is forced
        to be by prompt_cmd).

        :param prompt_cmd: Used when the prompt is not printed by default
        (happens on Windows) to print something that we can search for.

        :param prompt_change: Used to set the PS1/PS2 equivalents to
        something that is easier to tell apart from everything else,
        make sure it is a string with {0} and {1} in it for the
        PS1/PS2 fill-ins.

        See `metakernel.replwrap.REPLWrapper` for more details.

        """
        raise NotImplementedError

    def do_shutdown(self, restart):
        """
        Shut down the app gracefully, saving history.
        """
        try:
            self.wrapper.terminate()
        except Exception as e:
            self.Error(str(e))
        super(ProcessMetaKernel, self).do_shutdown(restart)

    def restart_kernel(self):
        """Restart the kernel"""
        self.wrapper = self.makeWrapper()


class DynamicKernel(ProcessMetaKernel):
    def __init__(self,
                 executable,
                 language,
                 mimetype="text/plain",
                 implementation="metakernel",
                 file_extension='txt',
                 orig_prompt=None,
                 prompt_change=None,
                 prompt_cmd=None,
                 extra_init_cmd=None,
                 stdin_prompt_regex=None):
        self.executable = executable
        self.orig_prompt = orig_prompt
        self.prompt_change = prompt_change
        self.prompt_cmd = prompt_cmd
        self.extra_init_cmd = extra_init_cmd
        self.stdin_prompt_regex = stdin_prompt_regex
        self.implementation = implementation
        self.language = language
        self.language_info['mimetype'] = mimetype
        self.language_info['language'] = language
        self.language_info['file_extension'] = file_extension
        super(DynamicKernel, self).__init__()

    def makeWrapper(self):
        return REPLWrapper(self.executable,
                           self.orig_prompt,
                           self.prompt_change,
                           prompt_cmd=self.prompt_cmd,
                           stdin_prompt_regex=self.stdin_prompt_regex,
                           extra_init_cmd=self.extra_init_cmd)

    """
    DynamicKernel(executable="bash",
                  orig_prompt=re.compile('[$#]'),
                  prompt_change=u"PS1='{0}' PS2='{1}' PROMPT_COMMAND=''",
                  prompt_cmd=None,
                  extra_init_cmd="export PAGER=cat",
                  implementation="bash_kernel",
                  language="bash",
                  mimetype="text/x-bash",
                  file_extension="sh")
    """


class BashKernel(ProcessMetaKernel):
    # Identifiers:
    implementation = 'bash_kernel'
    language = 'bash'
    language_info = {
        'mimetype': 'text/x-bash',
        'language': 'bash',
        # ------ If different from 'language':
        # 'codemirror_mode': 'language',
        # 'pygments_lexer': 'language',
        'file_extension': 'sh',
    }

    _banner = None
    @property
    def banner(self):
        if self._banner is None:
            self._banner = check_output(['bash', '--version']).decode('utf-8')
        return self._banner

    def makeWrapper(self):
        return bash()

if __name__ == '__main__':
    from IPython.kernel.zmq.kernelapp import IPKernelApp
    IPKernelApp.launch_instance(kernel_class=BashKernel)




import os
from metakernel.tests.utils import get_kernel, get_log_text, clear_log_text


def test_cd_magic():
    kernel = get_kernel()
    kernel.do_execute("%cd ~")
    assert os.getcwd() == os.path.expanduser('~'), os.getcwd()
    clear_log_text(kernel)
    kernel.do_execute('%cd')
    assert os.getcwd() in get_log_text(kernel)



# Copyright (c) Metakernel Development Team.
# Distributed under the terms of the Modified BSD License.

from metakernel import Magic
from IPython.display import Javascript

class JavascriptMagic(Magic):
    def line_javascript(self, code):
        """
        %javascript CODE - send code as JavaScript

        This line magic will execute the CODE on the line as
        JavaScript in the browser.

        Example:
            %javascript console.log("Print in the browser console")

        """
        jscode = Javascript(code)
        self.kernel.Display(jscode)

    def cell_javascript(self):
        """
        %%javascript - send contents of cell as JavaScript

        This cell magic will execute the contents of the cell as
        JavaScript in the browser.

        Example:
            %%javascript

            element.html("Hello this is <b>bold</b>!")

        """
        if self.code.strip():
            jscode = Javascript(self.code)
            self.kernel.Display(jscode)
            self.evaluate = False

def register_magics(kernel):
    kernel.register_magics(JavascriptMagic)




# Copyright (c) Metakernel Development Team.
# Distributed under the terms of the Modified BSD License.

from metakernel import Magic, option
import importlib
import logging

class KernelMagic(Magic):
    kernels = {}
    kernel_name = None

    @option(
        '-k', '--kernel_name', action='store', default="default",
        help='kernel name given to use for execution'
    )
    def line_kernel(self, module_name, class_name, kernel_name="default"):
        """
        %kernel MODULE CLASS [-k NAME] - construct a kernel for sending code.

        This line magic will contruct a kernel language so that you can
        communicate.

        Example:

            %kernel bash_kernel BashKernel -k bash

        Use `%kx` or `%%kx` to send code to the kernel.

        Also returns the kernel as output.
        """
        self.kernel_name = kernel_name
        module = importlib.import_module(module_name)
        class_ = getattr(module, class_name)
        kernel = class_()
        self.kernel.makeSubkernel(self.kernel)
        kernel.makeSubkernel(self.kernel)
        self.kernels[kernel_name] = kernel
        self.kernels[kernel_name].kernel = self.kernel
        self.retval = self.kernels[kernel_name]

    @option(
        '-k', '--kernel_name', action='store', default=None,
        help='kernel name given to use for execution'
    )
    def cell_kx(self, kernel_name=None):
        """
        %%kx [-k NAME] - send the cell code to the kernel.

        This cell magic will send the cell to be evaluated by
        the kernel. The kernel must have been created use the
        %%kernel magic.

        Returns the result of the execution as output.

        Example:

            %%kernel bash
            ls -al 

        Use `%kernel MODULE CLASS [-k NAME]` to create a kernel.
        """
        if kernel_name is None:
            kernel_name = self.kernel_name
        self.retval = self.kernels[kernel_name].do_execute_direct(self.code)
        self.evaluate = False

    @option(
        '-k', '--kernel_name', action='store', default=None,
        help='kernel name given to use for execution'
    )
    def line_kx(self, code, kernel_name=None):
        """
        %kx CODE [-k NAME] - send the code to the kernel.

        This line magic will send the CODE to the kernel
        for execution.

        Returns the result of the execution as output.

        Example:

            %kernel ls -al 

        Use `%kernel MODULE CLASS [-k NAME]` to create a kernel.
        """
        if kernel_name is None:
            kernel_name = self.kernel_name
        # make sure the code is sent as a string for execution
        code = str(code)
        self.retval = self.kernels[kernel_name].do_execute_direct(code)

    def post_process(self, retval):
        return self.retval

def register_magics(kernel):
    kernel.register_magics(KernelMagic)

def register_ipython_magics():
    from metakernel import IPythonKernel
    from IPython.core.magic import register_line_magic, register_cell_magic
    kernel = IPythonKernel()
    magic = KernelMagic(kernel)

    @register_line_magic
    def kernel(line):
        """
        line is module_name, class_name[, kernel_name]
        """
        if line.strip().count(" ") == 1:
            kernel_name = "default"
            module_name, class_name = [item.strip() for item in line.strip().split(" ", 1)]
        else:
            module_name, class_name, kernel_name = [item.strip() for item in line.strip().split(" ", 2)]
        magic.line_kernel(module_name, class_name, kernel_name)

    @register_cell_magic
    def kx(line, cell):
        """
        line is kernel_name, or "default"
        """
        if line.strip():
            module_name = line.strip()
        else:
            module_name = "default"
        magic.code = cell
        magic.cell_kx(module_name)




from metakernel.tests.utils import get_kernel, get_log_text


def test_connect_info_magic():
    kernel = get_kernel()
    kernel.do_execute("%connect_info")
    text = get_log_text(kernel)
    assert """{
  "stdin_port": UNKNOWN,
  "shell_port": UNKNOWN,
  "iopub_port": UNKNOWN,
  "hb_port": UNKNOWN,
  "ip": "UNKNOWN",
  "key": "UNKNOWN",
  "signature_scheme": "UNKNOWN",
  "transport": "UNKNOWN"
}

Paste the above JSON into a file, and connect with:
    $> ipython <app> --existing <file>
or, if you are local, you can connect with just:
    $> ipython <app> --existing UNKNOWN

or even just:
    $> ipython <app> --existing
if this is the most recent Jupyter session you have started.
""" in text, text

